/**************************************************************************************************
  LED_Driver.c

**************************************************************************************************/
#include "LED_Driver.h"
#include "USB_Driver.h"
#include "Buffers.h"
#include "built_in.h"


sbit CS_LED at GPIOD_ODR.B2;           // Chip select is on PD2


// Shadow registers are necessary since we can't read the max6951 registers
char Intensity = 0x07;                 // Start off with 1/2 intensity
char Configuration = 0x01;             // Normal operation (not shutdown)

/**************************************************************************************************
  Display_Init
  Sets up chip select for slot 1 of the shield board (PD2) and enables SPI3 on PC10, PC11, PC12.
  The SPI is set for 16-bit transfers.
**************************************************************************************************/
void Display_Init(void) {

     // Set PD2 (chip select) to output
     GPIO_Config(&GPIOD_BASE, _GPIO_PINMASK_2, _GPIO_CFG_DIGITAL_OUTPUT);
     CS_LED = 1;

     SPI3_Init_Advanced(_SPI_FPCLK_DIV64, _SPI_MASTER  | _SPI_16_BIT | _SPI_CLK_IDLE_LOW |
         _SPI_FIRST_CLK_EDGE_TRANSITION | _SPI_MSB_FIRST | _SPI_SS_DISABLE |
         _SPI_SSM_ENABLE | _SPI_SSI_1, &_GPIO_MODULE_SPI3_PC10_11_12);

     Display_Setup();
}

/**************************************************************************************************
  Display_Write
  Each transaction with MAX6951 includes a command address in bits 15:8 and data in bits 7:0.
  Communications are in one direction only. The MAX6951 has no input tied to MISO.
**************************************************************************************************/
void Display_Write(unsigned _data) {

     CS_LED = 0;
     SPI3_Write(_data);
     CS_LED = 1;
}

/**************************************************************************************************
  Display_Setup
  Turn on display and set for 8 digits HEX. Set intensity.
**************************************************************************************************/
void Display_Setup(void) {
     unsigned CmdDat;

     Hi(CmdDat) = CONFIGURATION;
     Lo(CmdDat) = Configuration;       // Configuration to normal
     Display_Write(CmdDat);

     Hi(CmdDat) = SCAN_LIMIT;
     Lo(CmdDat) = 0x07;                // Scan limit to 8 digits
     Display_Write(CmdDat);

     Hi(CmdDat) = DECODE_MODE;
     Lo(CmdDat) = 0xFF;                // Decode mode to 8-digit HEX
     Display_Write(CmdDat);

     Hi(CmdDat) = INTENSITY;
     Lo(CmdDat) = Intensity;           // Initial intensity is 1/2 brightness
     Display_Write(CmdDat);
}

/**************************************************************************************************
  DisplayAll
  Write a string of digits to the display.
**************************************************************************************************/
char DisplayAll(char *Msg) {
     char i;
     unsigned CmdDat;
     char *BufNum;

     if ((strlen(Msg) > 0) && (strlen(Msg) < 9)) {
         for (i=0;i<strlen(Msg);i++) {
              Hi(CmdDat) = DIG0P0P1 + i;
              Lo(CmdDat) = Msg[i];
              Display_Write(CmdDat);

              // Diagnostic
              BufNum = GetBuffer();
              sprintf(BufNum,"0x%04X",CmdDat);
              USB_PutQueue(BufNum);
         }
         return 0;
     } else {
         // Diagnostic
         BufNum = GetBuffer();
         sprintf(BufNum,"Length: %d",strlen(Msg));
         USB_PutQueue(BufNum);

         return 1;
     }
}

/**************************************************************************************************
  Set Individual Digits
  Write one digit to the display at a time. Digit can be 0 to 7, _data can be 0x00 to 0x0F.
  Characters '0' to '9' also work since the MAX6951 ignores the high order nibble.
**************************************************************************************************/
void SetDigit(char Digit, char _data, char DecPt) {
     unsigned CmdDat;
     char *BufNum;

     if ((Digit >= 0) && (Digit <=8)) {
         switch (Digit) {
             case 0:
                 Hi(CmdDat) = DIG0P0P1 + 0;
                 Lo(CmdDat) = _data;
                 if (DecPt) Lo(CmdDat) |= 0x80;
                 break;
             case 1:
                 Hi(CmdDat) = DIG0P0P1 + 1;
                 Lo(CmdDat) = _data;
                 if (DecPt) Lo(CmdDat) |= 0x80;
                 break;
             case 2:
                 Hi(CmdDat) = DIG0P0P1 + 2;
                 Lo(CmdDat) = _data;
                 if (DecPt) Lo(CmdDat) |= 0x80;
                 break;
             case 3:
                 Hi(CmdDat) = DIG0P0P1 + 3;
                 Lo(CmdDat) = _data;
                 if (DecPt) Lo(CmdDat) |= 0x80;
                 break;
             case 4:
                 Hi(CmdDat) = DIG0P0P1 + 4;
                 Lo(CmdDat) = _data;
                 if (DecPt) Lo(CmdDat) |= 0x80;
                 break;
             case 5:
                 Hi(CmdDat) = DIG0P0P1 + 5;
                 Lo(CmdDat) = _data;
                 if (DecPt) Lo(CmdDat) |= 0x80;
                 break;
             case 6:
                 Hi(CmdDat) = DIG0P0P1 + 6;
                 Lo(CmdDat) = _data;
                 if (DecPt) Lo(CmdDat) |= 0x80;
                 break;
             case 7:
                 Hi(CmdDat) = DIG0P0P1 + 7;
                 Lo(CmdDat) = _data;
                 if (DecPt) Lo(CmdDat) |= 0x80;
                 break;
         }
         Display_Write(CmdDat);
     } else {
         // Diagnostic
         BufNum = GetBuffer();
         sprintf(BufNum,"Digit error: %d",Digit);
         USB_PutQueue(BufNum);
     }
}

/**************************************************************************************************
  Turn On/Off Character Decoder for Any Digit.
  Each bit in the mask corresponds to a digit on the display. A 1 bit will decode the character
  a 0 bit will not decode the character. To blank a digit, the data reg for that digit must be
  0 and the decode bit for that position must be 0.

**************************************************************************************************/
void SetDecodeReg(char Mask) {
     unsigned CmdDat;

     Hi(CmdDat) = DECODE_MODE;
     Lo(CmdDat) = Mask;
     Display_Write(CmdDat);
}

/**************************************************************************************************
  Lower Intensity.
**************************************************************************************************/
void LowerIntensity(void) {
     unsigned CmdDat;
     char *BufNum;

     if (Intensity > 0) {
         Intensity -= 1;
         Hi(CmdDat) = INTENSITY;
         Lo(CmdDat) = Intensity;
         Display_Write(CmdDat);
     }
     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"Intensity: %d",Intensity);
     USB_PutQueue(BufNum);
}

/**************************************************************************************************
  Raise Intensity.
**************************************************************************************************/
void RaiseIntensity(void) {
     unsigned CmdDat;
     char *BufNum;

     if (Intensity < 0x0F) {
         Intensity += 1;
         Hi(CmdDat) = INTENSITY;
         Lo(CmdDat) = Intensity;
         Display_Write(CmdDat);
     }
     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"Intensity: %d",Intensity);
     USB_PutQueue(BufNum);
}

/**************************************************************************************************
  Configuration Register Functions.
  Datasheet:    D7    D6    D5    D4    D3    D2    D1    D0
                X     X     R     T     E     B     0     S

  It appears that there is an error in the MAX 6950/MAX6951 datasheet.
  Bit D1 seems to control blinking. Bit D2 and D3  have no effect.
  Blinking always occurs at the fast rate.
**************************************************************************************************/
void Shutdown(void) {
     unsigned CmdDat;
     char *BufNum;

     Hi(CmdDat) = CONFIGURATION;
     Configuration &= 0xFE;                      // Turn off S bit
     Lo(CmdDat) = Configuration;
     Display_Write(CmdDat);

     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"Configuration: %02X",Configuration);
     USB_PutQueue(BufNum);
}

void Normal(void) {
     unsigned CmdDat;
     char *BufNum;

     Hi(CmdDat) = CONFIGURATION;
     Configuration |= 0x01;                      // Turn on S bit
     Lo(CmdDat) = Configuration;
     Display_Write(CmdDat);

     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"Configuration: %02X",Configuration);
     USB_PutQueue(BufNum);
}

void BlinkSlow(void) {
     unsigned CmdDat;
     char *BufNum;

     Hi(CmdDat) = CONFIGURATION;
     Configuration &= 0xFB;                      // Turn off B bit (Bin: 1 1 1 1  1 0 1 1)
     Configuration |= 0x08;                      // Turn on E bit  (Bin: 0 0 0 0  1 0 0 0)
     Lo(CmdDat) = Configuration;
     Display_Write(CmdDat);

     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"Configuration: %02X",Configuration);
     USB_PutQueue(BufNum);
}

void BlinkFast(void) {
     unsigned CmdDat;
     char *BufNum;

     Hi(CmdDat) = CONFIGURATION;
     Configuration |= 0x0C;                      // Turn on B bit and E bit
     Lo(CmdDat) = Configuration;                 // (Bin: 0 0 0 0  1 1 0 0)
     Display_Write(CmdDat);

     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"Configuration: %02X",Configuration);
     USB_PutQueue(BufNum);
}

void BlinkOff(void) {
     unsigned CmdDat;
     char *BufNum;

     Hi(CmdDat) = CONFIGURATION;
     //Configuration = Configuration & 0xF7;     // Does not work as stated in the datasheet
     Configuration &= 0xFD;                      // Turn off E bit (Bin: 1 1 1 1  1 1 0 1)
     Lo(CmdDat) = Configuration;
     Display_Write(CmdDat);

     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"Configuration: %02X",Configuration);
     USB_PutQueue(BufNum);
}

void Test(char Mask) {
     unsigned CmdDat;
     char *BufNum;

     Hi(CmdDat) = CONFIGURATION;
     Configuration = Mask;                       // Set Configuration reg according to mask
     Lo(CmdDat) = Configuration;
     Display_Write(CmdDat);

     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"Configuration: %02X",Configuration);
     USB_PutQueue(BufNum);
}

void Increment(void) {
     unsigned CmdDat;
     char *BufNum;

     Hi(CmdDat) = CONFIGURATION;
     Configuration += 1;                         // Increment Configuration
     Lo(CmdDat) = Configuration;
     Display_Write(CmdDat);

     // Diagnostic
     BufNum = GetBuffer();
     sprintf(BufNum,"Configuration: %02X",Configuration);
     USB_PutQueue(BufNum);
}

void DispTest(char OnOff) {
     unsigned CmdDat;

     Hi(CmdDat) = DISPLAY_TEST;
     Lo(CmdDat) = OnOff;
     Display_Write(CmdDat);
}






/**************************************************************************************************
  EOF
**************************************************************************************************/